USE [eXtendedLogging]
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[sp_eXtended_Logging_Insert_LOG]
	@LogName nvarchar(200) = '',
	@RetentionDurationInMonth int,
	@LogDescription nvarchar(max) = '',
	@CustomField01 nvarchar(max) = '',
	@CustomField02 nvarchar(max) = ''
AS

BEGIN

SET NOCOUNT ON

DECLARE @ID uniqueidentifier
SET @ID = NEWID()

IF @RetentionDurationInMonth = ''
BEGIN
	SET @RetentionDurationInMonth = 1
END

INSERT INTO [eXtended_Logging_LOG]
	(
		[LogID],
		[Log_Name],
		[Retention_Duration_In_Month],
		[Start_DateTime],
		[LastUpdate_DateTime],
		[Log_Description],
		[Custom_Field_01],
		[Custom_Field_02]
	)
VALUES
	(
		@ID,
		@LogName,
		@RetentionDurationInMonth,
		current_timestamp,
		current_timestamp,
		@LogDescription,
		@CustomField01,
		@CustomField02
	)

SELECT
	convert(nvarchar(100),[LogID]) + ';' +
	[Log_Name] + ';' +
	convert(nchar(2),[Retention_Duration_In_Month]) + ';' +
	convert(nchar(19),isnull([Start_DateTime],0),120) + ';' +
	convert(nchar(19),isnull([LastUpdate_DateTime],0),120) + ';' +
	[Log_Description] + ';' +
	[Custom_Field_01] + ';' +
	[Custom_Field_02] as [LogData]

FROM 
	[eXtended_Logging_LOG]

WHERE
	[LogID] = @ID
END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[sp_eXtended_Logging_Insert_STEP]
	@LogID uniqueidentifier,
	@RunbookName nvarchar(200) = '',
	@RunbookServerName nvarchar(200) = '',
	@ActivityName nvarchar(200) = '',
	@ActivityStartDateTime datetime,
	@ActivityEndDateTime datetime,
	@Status nvarchar(50) = '',
	@EndStatusCode nvarchar(50) = '',
	@EndStatusDescription nvarchar(max) = '',
	@CustomField01 nvarchar(max) = '',
	@CustomField02 nvarchar(max) = '',
	@CustomField03 nvarchar(max) = '',
	@CustomField04 nvarchar(max) = ''
AS

BEGIN

SET NOCOUNT ON

DECLARE @ID uniqueidentifier
SET @ID = NEWID()

INSERT INTO [eXtended_Logging_STEP]
	(
		[StepID],
		[LogID],
		[Runbook_Name],
		[Runbook_Server_Name],
		[Activity_Name],
		[Activity_Start_DateTime],
		[Activity_End_DateTime],
		[Status],
		[End_Status_Code],
		[End_Status_Description],
		[Custom_Field_01],
		[Custom_Field_02],
		[Custom_Field_03],
		[Custom_Field_04]
	)
VALUES
	(
		@ID,
		@LogID,
		@RunbookName,
		@RunbookServerName,
		@ActivityName,
		@ActivityStartDateTime,
		@ActivityEndDateTime,
		@Status,
		@EndStatusCode,
		@EndStatusDescription,
		@CustomField01,
		@CustomField02,
		@CustomField03,
		@CustomField04
	)

UPDATE [eXtended_Logging_LOG]
SET [LastUpdate_DateTime] = current_timestamp
WHERE [LogID] = @LogID
	
SELECT
	convert(nvarchar(100),[StepID]) + ';' +
	convert(nvarchar(100),[LogID]) + ';' +
	[Runbook_Name] + ';' +
	[Runbook_Server_Name] + ';' +
	[Activity_Name] + ';' +
	convert(nchar(19),isnull([Activity_Start_DateTime],0),120) + ';' +
	convert(nchar(19),isnull([Activity_End_DateTime],0),120) + ';' +
	[Status] + ';' +
	[End_Status_Code] + ';' +
	[End_Status_Description] + ';' +
	[Custom_Field_01] + ';' +
	[Custom_Field_02] + ';' +
	[Custom_Field_03] + ';' +
	[Custom_Field_04] as [LogData]
FROM 
	[eXtended_Logging_STEP]
WHERE
	[StepID] = @ID
END

GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[sp_eXtended_Logging_Select_GETLOG]
	@LogID uniqueidentifier
AS

BEGIN

SET NOCOUNT ON

SELECT 
		convert(nvarchar(100),[LogID]) + ';' +
		[Log_Name] + ';' +
		convert(nchar(2),[Retention_Duration_In_Month]) + ';' +
		convert(nchar(19),isnull([Log_Start_DateTime],0),120) + ';' +
		convert(nchar(19),isnull([Log_LastUpdate_DateTime],0),120) + ';' + 
		[Log_Description] + ';' +
        [Log_Custom_Field_01] + ';' +
		[Log_Custom_Field_02] + ';' +
		convert(nvarchar(100),[StepID]) + ';' +
		[Runbook_Name] + ';' +
		[Runbook_Server_Name] + ';' +
		[Activity_Name] + ';' +
		convert(nchar(19),isnull([Activity_Start_DateTime],0),120) + ';' +
		convert(nchar(19),isnull([Activity_End_DateTime],0),120) + ';' +
		[Status] + ';' +
		[End_Status_Code] + ';' +
		[End_Status_Description] + ';' +
		[Step_Custom_Field_01] + ';' +
		[Step_Custom_Field_02] + ';' +
		[Step_Custom_Field_03] + ';' +
		[Step_Custom_Field_04] as [LogData]		
FROM eXtended_Logging_GETLOG
WHERE [LogID] = @LogID

END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[sp_eXtended_Logging_Purge] 
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @CURRENTDATE datetime
	DECLARE @NumberOfLogRemoved int
	
	SET @CURRENTDATE = GETDATE()

	SET @NumberOfLogRemoved = (SELECT COUNT(LogID) AS number_Of_Log_Removed FROM eXtended_Logging_LOG
	WHERE LastUpdate_DateTime < (DATEADD(MONTH,-([Retention_Duration_In_Month]),@CURRENTDATE)))
	
	DELETE FROM eXtended_Logging_STEP
	WHERE LogID IN (SELECT LogID FROM eXtended_Logging_LOG
	WHERE LastUpdate_DateTime < (DATEADD(MONTH,-([Retention_Duration_In_Month]),@CURRENTDATE)))
	
	DELETE FROM eXtended_Logging_LOG
	WHERE LastUpdate_DateTime < (DATEADD(MONTH,-([Retention_Duration_In_Month]),@CURRENTDATE))

	SELECT convert(nvarchar(50),@NumberOfLogRemoved) AS [LogData]
END
GO